/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 David Dribin.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Ant" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.dribin.dave.ant.j2me.taskdefs;

import java.io.*;

import org.apache.tools.ant.*;
import org.apache.tools.ant.util.*;
import org.apache.tools.ant.taskdefs.Execute;
import org.apache.tools.ant.types.Commandline;

import org.dribin.dave.ant.j2me.util.*;

public class Preverify extends Task
{
    public void init()
    {
        StringBuffer    preverifyExecutablePath;
        String          osPath;
        String          kjavaHome;

        super.init();

        mOsNotSupported = false;
        mKvmHelper  = new KvmHelper(project);
        osPath      = getOsPath();
        mPreverifyExecutable    = null;
        kjavaHome   = mKvmHelper.getKjavaHome();
        
        if (osPath == null)
        {
            mOsNotSupported = true;
            return;
        }

        if (kjavaHome == null)
        {
            return;
        }

        preverifyExecutablePath = new StringBuffer(kjavaHome);
        preverifyExecutablePath.append(File.separator);
        preverifyExecutablePath.append("bin").append(File.separator);
        preverifyExecutablePath.append(osPath).append(File.separator);
        preverifyExecutablePath.append("preverify");

        mPreverifyExecutable    = new File(preverifyExecutablePath.toString());

        if (! mPreverifyExecutable.isFile())
        {
            mPreverifyExecutable    = null;
        }
    }

    private String getOsPath()
    {
        String  osName;
        String  osPath;

        osName  = project.getProperty("os.name").toLowerCase();

        if (osName.indexOf("linux") >= 0)
        {
            osPath  = "linux";
        }
        else if (osName.indexOf("windows") >= 0)
        {
            osPath  = "win32";
        }
        else if (osName.indexOf("solaris") >= 0)
        {
            osPath  = "solaris";
        }
        else
        {
            // Preverifier not supported under other OSes
            osPath  = null;
        }

        return osPath;
    }
    
    public void setUnverifieddir(File unverifiedDir)
    {
        mUnverifiedDir  = unverifiedDir;
    }

    public void setVerifieddir(File verifiedDir)
    {
        mVerifiedDir    = verifiedDir;
    }

    public void execute() throws BuildException
    {
        Execute     command;
        Commandline commandLine;

        if (mOsNotSupported)
        {
            throw new BuildException("Preverify not supported under OS: " +
                                     project.getProperty("os.name"), location);
        }

        if (mPreverifyExecutable == null)
        {
            throw new BuildException("Preverify exectable not found", location);
        }
        
        if (mUnverifiedDir == null)
        {
            throw new BuildException("unverifieddir attribute must be set",
                                     location);
        }
        
        if (! mUnverifiedDir.isDirectory())
        {
            throw new BuildException("unverifieddir \"" + mUnverifiedDir  +
                                     "\" does not exist or " +
                                     "is not a directory",
                                     location);
        }
        
        if (mVerifiedDir == null)
        {
            throw new BuildException("verifieddir attribute must be set",
                                     location);
        }
        
        if (! mVerifiedDir.isDirectory())
        {
            throw new BuildException("verifieddir \"" + mVerifiedDir  +
                                     "\" does not exist or " +
                                     "is not a directory",
                                     location);
        }
        
        if (areAllFilesUpToDate())
        {
            return;
        }

        try
        {
            commandLine = new Commandline();
            commandLine.setExecutable(mPreverifyExecutable.getPath());
            commandLine.createArgument().setValue("-classpath");
            commandLine.createArgument().setPath(
                mKvmHelper.getKjavaClassPath());
            commandLine.createArgument().setValue("-d");
            commandLine.createArgument().setFile(mVerifiedDir);
            commandLine.createArgument().setFile(mUnverifiedDir);
        
            log("Preverifying classes to " + mVerifiedDir);
            log("Executing: " + commandLine, Project.MSG_VERBOSE);
            command = new Execute();
            command.setCommandline(commandLine.getCommandline());
            command.execute();
            if (command.getExitValue() != 0)
            {
                throw new BuildException(
                    "Preverify failed, messages should have been provided");
            }
        }
        catch (IOException ioe)
        {
            throw new BuildException(ioe, location);
        }
    }

    private boolean areAllFilesUpToDate()
    {
        String[]    filesToVerify;
        SourceFileScanner   scanner;

        scanner = new SourceFileScanner(this);
        filesToVerify   = scanner.restrict(getUnverifiedFiles(),
                                           mUnverifiedDir, mVerifiedDir,
                                           new IdentityMapper());
        log(filesToVerify.length + " file(s) to verify", Project.MSG_VERBOSE);
        if (filesToVerify.length == 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    private String[] getUnverifiedFiles()
    {
        DirectoryScanner    ds;
        String[]    includes;

        ds  = new DirectoryScanner();
        includes    = new String[1];
        includes[0] = "**" + File.separator + "*.class";
        ds.setIncludes(includes);
        ds.setBasedir(mUnverifiedDir);
        ds.scan();

        return ds.getIncludedFiles();
    }

    private KvmHelper               mKvmHelper;
    private File                    mPreverifyExecutable;
    private File                    mUnverifiedDir;
    private File                    mVerifiedDir;
    private boolean                 mOsNotSupported;
}
